/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of tallyphot
 *
 * tallyphot is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * tallyphot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stddef.h>

#include <suplib/times.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>
#include <bpipe/bpipe_axaf.h>

#include "main.h"
#include "common.h"

/* Routine to get the data pointer for a named data packet field */

static char *
dpktf_data( BPipe *bpipe, char *name, void *data )
{
  DpktField *field = bpipe_dpktf( bpipe, name );

  if ( field == NULL )
    tf_die( "%s isn't in input data stream", name);

  return bpipe_dpktf_data( field, data );

}

void
process(void)
{
  BPipe *bpipe;
  BPipeOutput *bpo;

  void *data;

  double *weight;

  size_t dpkt_core_size;
  size_t n;

  Time cum_time;

  double wt;

  tf_enter("tallyphot");

  /* Setup bpipe input and output streams */

  if ( NULL == ( bpipe = bpipe_new() ) )
    tf_die( "couldn't open bpipe: %s", bpipe_strerror(bpipe_errno));

  if ( bpipe_input(bpipe, c.input) )
    tf_die( "couldn't open input file `%s': %s", c.input,
	    bpipe_strerror(bpipe_errno));

  if ( NULL == ( bpo = bpipe_output( bpipe, c.output ) ) )
    tf_die( "couldn't open output file `%s': %s", c.output,
	    bpipe_strerror(bpipe_errno));

  if (NULL == (data = bpipe_map_alloc(bpipe,1, &dpkt_core_size)))
    tf_exit( ExitERR_alloc, "unable to allocate memory for data packet: %s",
	    bpipe_strerror(bpipe_errno));

  weight   = (double *)   dpktf_data( bpipe, BP_AXAF_phot_wt, data);

  if ( bpipe_write_hdr(bpipe) )
    tf_die( "bpipe_write_hdr: %s",bpipe_strerror(bpipe_errno));

  /* open up the input photon stream */

  bpipe_errno = BPNOERROR;

  n = 0;

  wt = 0.0;

  startTime(&cum_time);
  startTime(NULL);

  while( bpipe_read_dpkts( bpipe, data, 1 ) == 1) 
  {
    n++;
    if ( *weight )
      wt += *weight;           

    if (c.outcnt && !(n%c.outcnt))
    {
      if (*weight)
        tf_message("%d %f\n", n, wt);
      else
        tf_message("%d\n", n);

      if ( c.time )
	tf_message(" %s\n", stringTime(elapsedTime(NULL)));

      startTime(NULL);
    }

    if( bpipe_write_dpkt( bpipe, data, bpo) )
      tf_die( "error writing photon in tallyphot %d", n);
  }

  if ( bpipe_errno != BPNOERROR )
    tf_die( bpipe_strerror( bpipe_errno ) );

  if (*weight)
    tf_message("final tally: %d %f\n", n, wt);
  else
    tf_message("final tally: %d\n", n);

  if (c.time)
    tf_message(" %s\n", stringTime(elapsedTime(&cum_time)));

  bpipe_delete( bpipe );

  tf_leave();
}

